
def get_paginated_response(session, url, params={}, **kwargs):
    """
    Get the results of all pages of a response. Returns an iterator
    that returns each of the items.
    """
    while url is not None:
        data = session.get(url, params=params, **kwargs)
        for result in data.get('results', []):
            yield result
        url = data.get('next', None)
        if url is not None:
            # We remove part of the url that the session already has
            url = url.replace(session.url, '')
        # params are included in the next url
        params = {}



from .seed_services import SeedServicesApiClient
from .utils import get_paginated_response


class ServiceRatingApiClient(SeedServicesApiClient):
    """
    Client for Service Rating Service.
    """

    # Invites
    def get_invites(self, params=None):
        return {"results": get_paginated_response(self.session,
                '/invite/', params=params)}

    def get_invite(self, invite_id):
        return self.session.get('/invite/%s/' % invite_id)

    def create_invite(self, invite):
        return self.session.post('/invite/', data=invite)

    def update_invite(self, invite_id, data=None):
        return self.session.patch('/invite/%s/' % invite_id, data=data)

    def delete_invite(self, invite_id):
        # Ratings should be deleted first for FK reasons
        self.session.delete('/invite/%s/' % invite_id)
        return {"success": True}

    # Ratings
    def get_ratings(self, params=None):
        return {"results": get_paginated_response(self.session,
                '/rating/', params=params)}

    def get_rating(self, rating_id):
        return self.session.get('/rating/%s/' % rating_id)

    def create_rating(self, rating):
        return self.session.post('/rating/', data=rating)

    def update_rating(self, rating_id, data=None):
        return self.session.patch('/rating/%s/' % rating_id, data=data)

    def delete_rating(self, rating_id):
        self.session.delete('/rating/%s/' % rating_id)
        return {"success": True}



from .seed_services import SeedServicesApiClient
from .utils import get_paginated_response


class MessageSenderApiClient(SeedServicesApiClient):
    """
    Client for Message Sender Service.

    :param str auth_token:
        An access token.

    :param str api_url:
        The full URL of the API.

    :param JSONServiceClient session:
        An instance of JSONServiceClient to use

    """

    def create_outbound(self, payload):
        return self.session.post('/outbound/', data=payload)

    def get_outbounds(self, params=None):
        return {"results": get_paginated_response(self.session, '/outbound/',
                params=params)}

    def create_inbound(self, payload):
        return self.session.post('/inbound/', data=payload)

    def get_inbounds(self, params=None):
        return {"results": get_paginated_response(self.session, '/inbound/',
                params=params)}

    def get_failed_tasks(self, params=None):
        return {"results": get_paginated_response(self.session,
                '/failed-tasks/', params=params)}

    def requeue_failed_tasks(self):
        return self.session.post('/failed-tasks/')



from .seed_services import SeedServicesApiClient
from .utils import get_paginated_response


class SchedulerApiClient(SeedServicesApiClient):
    """
    Client for Scheduler Service.
    """

    def get_schedules(self, params=None):
        return {"results": get_paginated_response(self.session, '/schedule/',
                params=params)}

    def get_schedule(self, schedule_id):
        return self.session.get('/schedule/%s/' % schedule_id)

    def create_schedule(self, schedule):
        return self.session.post('/schedule/', data=schedule)

    def update_schedule(self, schedule_id, schedule):
        return self.session.patch('/schedule/%s/' % schedule_id,
                                  data=schedule)

    def delete_schedule(self, schedule_id):
        return self.session.delete('/schedule/%s/' % schedule_id)

    def get_failed_tasks(self, params=None):
        return {"results": get_paginated_response(self.session,
                '/failed-tasks/', params=params)}

    def requeue_failed_tasks(self):
        return self.session.post('/failed-tasks/')



from .seed_services import SeedServicesApiClient
from .utils import get_paginated_response


class StageBasedMessagingApiClient(SeedServicesApiClient):
    """
    Client for Stage Based Messaging Service.

    :param str auth_token:
        An access token.

    :param str api_url:
        The full URL of the API.

    :param JSONServiceClient session:
        An instance of JSONServiceClient to use

    :param HTTPServiceClient session_https:
        An instance of HTTPServiceClient to use

    """

    def get_schedules(self, params=None):
        return {"results": get_paginated_response(self.session, '/schedule/',
                params=params)}

    def get_schedule(self, schedule_id):
        return self.session.get('/schedule/%s/' % schedule_id)

    def get_messagesets(self, params=None):
        return {"results": get_paginated_response(self.session, '/messageset/',
                params=params)}

    def get_messageset(self, messageset_id):
        return self.session.get('/messageset/%s/' % messageset_id)

    def get_messageset_languages(self):
        return self.session.get('/messageset_languages/')

    def get_subscription(self, subscription):
        return self.session.get('/subscriptions/%s/' % subscription)

    def get_subscriptions(self, params=None):
        return {"results": get_paginated_response(self.session,
                '/subscriptions/', params=params)}

    def get_messages(self, params=None):
        return {"results": get_paginated_response(self.session, '/message/',
                params=params)}

    def get_message(self, message_id):
        return self.session.get('/message/%s/' % message_id)

    def delete_message(self, message_id):
        return self.session.delete('/message/%s/' % message_id)

    def update_message(self, message_id, data=None):
        return self.session.patch(
            '/message/{0}/'.format(message_id),
            data=data)

    def delete_binarycontent(self, binarycontent_id):
        return self.session.delete('/binarycontent/%s/' % binarycontent_id)

    def create_message(self, message):
        return self.session.post('/message/', data=message)

    def create_binarycontent(self, content):
        return self.session_http.post('/binarycontent/', files=content).json()

    def update_subscription(self, subscription, data=None):
        return self.session.patch('/subscriptions/%s/' % subscription,
                                  data=data)

    def create_subscription(self, subscription):
        return self.session.post('/subscriptions/', data=subscription)

    def resend_subscription(self, subscription):
        return self.session.post('/subscriptions/%s/resend' % subscription)

    def get_failed_tasks(self, params=None):
        return {"results": get_paginated_response(self.session,
                '/failed-tasks/', params=params)}

    def requeue_failed_tasks(self):
        return self.session.post('/failed-tasks/')



from demands import HTTPServiceClient, JSONServiceClient
from requests.adapters import HTTPAdapter

from .__version__ import __version__ as client_version


class SeedHTTPAdapter(HTTPAdapter):
    """
    HTTPAdapter child class to implement global timeouts
    """

    def __init__(self, timeout=None, *args, **kwargs):
        self.timeout = timeout
        super(SeedHTTPAdapter, self).__init__(*args, **kwargs)

    def send(self, *args, **kwargs):
        kwargs['timeout'] = self.timeout
        return super(SeedHTTPAdapter, self).send(*args, **kwargs)


class SeedServicesApiClient(object):
    """
    Base API client for seed services.

    :param str auth_token:
        An access token.

    :param str api_url:
        The full URL of the API.

    :param JSONServiceClient session:
        An instance of JSONServiceClient to use

    :param HTTPServiceClient session_https:
        An instance of HTTPServiceClient to use

    :param retries:
        (optional) The number of times to retry an HTTP request

    :param timeout:
        (optional) The number of seconds for a request to timeout,
        defaults to 65 seconds

    """

    def __init__(self, auth_token, api_url, session=None, session_http=None,
                 retries=0, timeout=65):

        headers = {
            'Authorization': 'Token ' + auth_token,
            'User-Agent': 'seed-services-client v{0}'.format(client_version),
        }

        if session is None:
            session = JSONServiceClient(url=api_url,
                                        headers=headers)
        self.session = session

        if session_http is None:
            session_http = HTTPServiceClient(url=api_url,
                                             headers=headers)

        self.session_http = session_http

        http_adapter_kwargs = {}

        if retries > 0:
            http_adapter_kwargs['max_retries'] = retries

        if timeout is not None:
            http_adapter_kwargs['timeout'] = timeout

        http = SeedHTTPAdapter(**http_adapter_kwargs)
        https = SeedHTTPAdapter(**http_adapter_kwargs)
        self.session.mount('http://', http)
        self.session.mount('https://', https)
